import re

class Video:
    def __init__(self, soup):
        self.soup = soup
        self.url = self.__getURL()

    def __getURL(self):
        download_link = ""
        # Find the download link in the HTML
        # The .mp4 link is found in an input where the id starts with "edit-debate-file-options-download-"
        # Then the link is found in the value attribute of that input
        
        download_link = self.soup.find(
            "input", {"id": re.compile(r"^edit-debate-file-options-download-")}
        )

        if download_link and type(download_link) == str:
            # Remove the "download_" prefix from the link
            download_link = download_link.replace("download_", "")
        else:
            try:
                # Find an input id that starts with "edit-debate-file-options-azure-httpsdebatgemist"
                download_link = self.soup.find(
                    "input", {"id": re.compile(r"^edit-debate-file-options-azure-httpsdebatgemist")}
                )
                download_link = download_link["value"]
                download_link = download_link.replace("azure_", "")
            except:
                pass

        return download_link


import re
import requests
from nltk.tokenize import RegexpTokenizer

class Subtitle:
    def __init__(self, soup, url):
        self.soup = soup
        self.url = url
        self.raw = self.get()
        self.text = self.__getText()
        self.tokenized = self.__tokenize()

    def getFilename(self):
        # Find the subtitle link in the HTML
        # Find the input tag where the id starts with "edit-debate-file-options-" and the value ends with ".srt"
        subtitle_link = self.soup.find(
            "input",
            {
                "id": re.compile(r"^edit-debate-file-options-"),
                "value": re.compile(r".srt$"),
            },
        )

        try:
            subtitle_link = subtitle_link["value"]
        except:
            subtitle_link = ""

        return subtitle_link

    def __getText(self):
        clean_text = self.cleanSubtitle(self.raw)

        return clean_text

    def cleanSubtitle(self, raw_subtitle):
        clean_text = raw_subtitle

        # Remove the index number from the subtitle
        clean_text = re.sub(r"^\d+\s", "", clean_text, flags=re.MULTILINE)

        # Remove the timecode from the subtitle
        clean_text = re.sub(r"^\d\d:\d\d:\d\d,\d\d\d --> \d\d:\d\d:\d\d,\d\d\d\s", "", clean_text, flags=re.MULTILINE)

        # Remove the empty lines
        clean_text = re.sub(r"^\s*$", "", clean_text, flags=re.MULTILINE)

        return clean_text

    def __tokenize(self):
        tokenizer = RegexpTokenizer(r"\w+")
        tokens = tokenizer.tokenize(self.text.lower())

        return tokens

    
    def getTextFromXtoY(self, x, y):
        raw = self.getFromXtoY(x, y)
        clean_text = self.cleanSubtitle(raw)

        return clean_text

    # Get subtitles from second x to second y
    def getFromXtoY(self, x, y):
        # Get the raw text
        raw_text = self.raw

        # Split the text into a list of lines
        lines = raw_text.splitlines()

        # Create a list to store the lines that are between x and y
        lines_in_range = []

        # Loop through the lines
        for line in lines:
            # Check if the line is a timecode
            # Example: 02:38:30,330 --> 02:38:35,620
            if re.match(r"^\d\d:\d\d:\d\d,\d\d\d --> \d\d:\d\d:\d\d,\d\d\d$", line):
                # Split the timecode into two parts
                timecode = line.split(" --> ")

                # Get the start and end time
                start_time = timecode[0]
                end_time = timecode[1]

                # Convert the start and end time to seconds
                start_time = self.__timecodeToSeconds(start_time)
                end_time = self.__timecodeToSeconds(end_time)

                # Check if the start and end time are between x and y
                if start_time >= x and end_time <= y:
                    # Add the timecode to the list
                    lines_in_range.append(line)
                else:
                    # Check if the start time is between x and y
                    if start_time >= x and start_time <= y:
                        # Add the timecode to the list
                        lines_in_range.append(line)

                    # Check if the end time is between x and y
                    if end_time >= x and end_time <= y:
                        # Add the timecode to the list
                        lines_in_range.append(line)
                if start_time > y:
                    break
            else:
                # Check if the line is between x and y
                if len(lines_in_range) > 0:
                    # Add the line to the list
                    lines_in_range.append(line)

        # Join the lines into a string
        text = "\n".join(lines_in_range)

        return text

    
    def __timecodeToSeconds(self, timecode):
        # Split the timecode into hours, minutes, seconds and milliseconds
        timecode = timecode.split(":")
        hours = int(timecode[0])
        minutes = int(timecode[1])
        seconds = int(timecode[2].split(",")[0])
        milliseconds = int(timecode[2].split(",")[1])

        # Convert the timecode to seconds
        seconds = hours * 3600 + minutes * 60 + seconds + milliseconds / 1000

        return seconds

    def get(self):
        subtitle_link = self.getFilename()

        # Get the form-build-id from the HTML
        form_build_id = self.soup.find("input", {"name": "form_build_id"})["value"]

        # Create a POST request with the following form-data
        # form_build_id: form_build_id
        # form_id: "debatgemist_download_video_form"
        # op: "Download"
        # debate_file_options: subtitle_link
        data = {
            "form_build_id": form_build_id,
            "form_id": "debatgemist_download_video_form",
            "op": "Download",
            "debate_file_options": subtitle_link,
        }

        # Request the subtitle file
        r = requests.post(self.url, data=data)

        if r.status_code != 200:
            print("Error requesting URL")
            exit(1)

        # Return the subtitle file as string
        return r.text


import re
import requests
import csv
from bs4 import BeautifulSoup
from .debate import Debate
from .enums.debate_type import DebateType
from .enums.sort_direction import SortDirection
from .enums.sort_type import SortType


class Search:
    def __init__(self, query, start_date: str = "", end_date: str = "", debate_type = DebateType.ANY, sort = SortType.RELEVANCE, sort_direction = SortDirection.DESC, limit = 10) -> None:
        self.base_url = "https://debatgemist.tweedekamer.nl"
        self.result = self.__getDebates(query, start_date, end_date, debate_type, sort, sort_direction, limit)

    def __checkAndGenerateSearchUrl(self, query, start_date, end_date, debate_type, sort = SortType.RELEVANCE, sort_direction = SortDirection.DESC, limit = 10) -> str:
        base_url = "https://debatgemist.tweedekamer.nl"

        search_url = base_url + "/zoeken?search_api_views_fulltext={}"
        search_url = search_url.format(query)

        dates_added = False

        if start_date and end_date:
            # Check date format
            if not re.match(r"^\d{4}-\d{2}-\d{2}$", start_date):
                raise ValueError("Invalid start date, format should be YYYY-MM-DD")

            if not re.match(r"^\d{4}-\d{2}-\d{2}$", end_date):
                raise ValueError("Invalid end date, format should be YYYY-MM-DD")

            search_url = search_url + "&f[0]=created:[{}T00:00:00Z TO {}T00:00:00Z]"
            search_url = search_url.format(start_date, end_date)

            dates_added = True
        
        if debate_type != DebateType.ANY:
            if dates_added:
                search_url = search_url + "&f[1]={}"
            else:
                search_url = search_url + "&f[0]={}"

            search_url = search_url.format(debate_type.value)

        search_url = search_url + "&sort={}&order"
        search_url = search_url.format(sort.value, sort_direction.value)

        assert limit > 0
        assert search_url != base_url

        return search_url

    def __getSoup(self, search_url) -> BeautifulSoup:
        r = requests.get(search_url)
        assert r.status_code == 200

        soup = BeautifulSoup(r.text, "html.parser")
        return soup

    # Based on a text query returns a list of debate urls
    def __getDebates(self, query, start_date = "", end_date = "", debate_type = DebateType.ANY, sort = SortType.RELEVANCE, sort_direction = SortDirection.DESC, limit = 10):
        self.__checkAndGenerateSearchUrl(query, start_date, end_date, debate_type, sort, sort_direction, limit)
        debate_urls = self.__getDebateUrls(query, start_date, end_date, debate_type, sort, sort_direction, limit)
        if debate_urls:
            return [Debate(url) for url in debate_urls]
        else:
            return []
        
    
    def to_csv(self, filename, separate_speakers = True, delimiter = ","):
        if delimiter not in [";", ","]:
            raise ValueError("Invalid delimiter, only ';' and ',' are supported")

        with open('{}.csv'.format(filename), 'w', newline='') as file:
            writer = csv.writer(file, delimiter=delimiter, quoting=csv.QUOTE_ALL)

            if separate_speakers:
                writer.writerow(["title", "date", "url", "video_url", "speaker", "party", "tokenized", "subtitle"])
            else:
                writer.writerow(["title", "date", "url", "video_url", "tokenized", "subtitle", "text"])

            for debate in self.result:
                if not separate_speakers:
                    writer.writerow([debate.title, debate.date, debate.url, debate.video.url, debate.subtitle.tokenized, debate.subtitle.raw, debate.subtitle.text])
                    continue

                for speaker in debate.speakers:
                    writer.writerow([debate.title, debate.date, debate.url, debate.video.url, speaker.name, speaker.party, speaker.speach.tokenized, speaker.speach.subtitle])
            
            file.close()
                        


    def __getDebateUrls(self, query, start_date = None, end_date = None, debate_type = DebateType.ANY, sort = SortType.RELEVANCE, sort_direction = SortDirection.DESC, limit = 10):
        done = False
        search_url = self.__checkAndGenerateSearchUrl(query, start_date, end_date, debate_type, sort, sort_direction, limit)
        debates = []

        while not done:
            soup = self.__getSoup(search_url)

            # Find all a href tags where the class is "video-thumb"
            temp_debates = soup.find_all("a", {"class": "video-thumb"})
            temp_debates = [self.base_url + debate["href"] for debate in temp_debates]

            for debate in temp_debates:
                if len(debates) >= limit:
                    done = True
                    break

                debates.append(debate)

            # Find the next page link, an a tag with the title "Ga naar volgende pagina"
            next_page = soup.find("a", {"title": "Ga naar volgende pagina"})

            if next_page:
                search_url = self.base_url + next_page["href"]
            else:
                done = True

        return debates


from nltk.tokenize import RegexpTokenizer

class Speach:
    def __init__(self, debate, speaker):
        self.text = ""
        self.subtitle = ""
        self.tokenized = []

        try:
            self.text = debate.subtitle.getTextFromXtoY(speaker.start_time, speaker.end_time)
            self.subtitle = debate.subtitle.getFromXtoY(speaker.start_time, speaker.end_time)
            self.tokenized = self.__tokenize()
        except:
            pass


    def __tokenize(self):
        tokenizer = RegexpTokenizer(r"\w+")
        tokens = tokenizer.tokenize(self.text.lower())

        return tokens


__version__ = "0.1.1"

from .debate import Debate
from .search import Search
from .enums.debate_type import DebateType
from .enums.sort_direction import SortDirection
from .enums.sort_type import SortType


from datetime import datetime
import json
import requests
import re
from bs4 import BeautifulSoup
from .video import Video
from .subtitle import Subtitle
from .speaker import Speaker

class Debate:
    def __init__(self, url) -> None:
        self.url = url
        self.__soup = self.__getSoup()
        self.title = self.__getTitle()
        self.subtitle = Subtitle(self.__soup, self.url)
        self.video = Video(self.__soup)
        self.date = self.__getDate()
        self.readable_date = self.__getReadableDate()
        self.speakers = self.__getSpeakers()

    def __getSpeakersFromJSON(self):
        # Find the video tag with id "ys-videojs" and get the data-setup attribute
        # The data-setup attribute contains a JSON string with the speakers
        speakers_json = self.__soup.find("video", {"id": "ys-videojs"})

        if speakers_json is None:
            return []
        
        speakers_json = speakers_json["data-setup"]

        # Convert the JSON string to a Python dictionary
        speakers = json.loads(speakers_json)
        return speakers.get("plugins").get("chapterdock").get("chapters", [])

    def __getSpeakers(self):
        speakers = [Speaker(self, data) for data in self.__getSpeakersFromJSON()]
        return speakers


    def __getSoup(self) -> BeautifulSoup:
        if not re.match(r"^https?://", self.url):
            # Throw an error if the URL doesn't start with http or https
            raise ValueError("Invalid URL")

        # Request the URL
        r = requests.get(self.url)
        assert r.status_code == 200

        # Parse the HTML
        soup = BeautifulSoup(r.text, "html.parser")

        return soup

    def __getTitle(self) -> str:
        title = ""

        try:
            # Get the title of the video
            title = self.__soup.find("title").text

            # Remove the " | Debat Gemist" part of the title
            title = title.replace(" | Debat Gemist", "")
        except:
            pass

        return title
    
    def __getDate(self) -> datetime:
        date = None

        try:
            # Find the time tag and get the datetime attribute
            date = self.__soup.find("time").get("datetime", None)

            # String to datetime
            date = datetime.strptime(date, "%Y-%m-%d")
        except:
            pass

        return date

    def __getReadableDate(self) -> str:
        date = ""

        try:
            # Find the time tag and get the inner text
            date = self.__soup.find("time").text
        except:
            pass

        return date


from .speach import Speach

class Speaker:
    def __init__(self, parent, data):
        self.data = data
        self.name = self.__getName()
        self.short_name = self.__getShortName()
        self.party = self.__getParty()
        self.function = self.__getFunction()
        self.start_time = self.__getStartTime()
        self.end_time = self.__getEndTime()
        self.image_url = self.__getImageUrl()
        self.duration = abs(self.end_time - self.start_time)
        self.speach = Speach(parent, self)
    
    def __getName(self):
        return self.data.get("name", "")
    
    def __getShortName(self):
        return self.data.get("name_short", "")

    def __getParty(self):
        full_function = self.data.get("parliamentary_party", "")
        
        try:
            party = full_function.split(" - ")[0]
        except:
            party = ""

        return party

    def __getFunction(self):
        full_function = self.data.get("parliamentary_party", "")

        try:
            function = full_function.split(" - ")[1]
        except:
            function = ""

        return function
    

    def __getStartTime(self):
        start = self.data.get("start", 0)
        start = max(start, 0)
        start = int(start)

        return start
    

    
    def __getEndTime(self):
        end = self.data.get("end", 0)
        end = max(end, 0)
        end = int(end)

        return end

    def __getImageUrl(self):
        return self.data.get("speaker_image", "")


